package com.ibm.jikes.skij; 
import java.util.Hashtable;

/* This file is part of Skij.
 * Author: Michael Travers (mt@watson.ibm.com)
 * 
 * Licensed Materials - See the file license.txt.
 * (c) Copyright IBM Corp. 1997, 1998. All rights reserved.
 */

public class ProcEnvironment extends Environment {

  public Object names = Nil.nil; // these two probably should not be public once reflection is fixed
  public Cons values = Nil.nil;
  public Environment dynamicParent = null;
  
  public ProcEnvironment() {
  }

  public ProcEnvironment(Environment p) {
    parent = p;
  }

  public ProcEnvironment(Environment p, Environment dp) {
    // here's a kludge to prevent frame build-up
    if (dp.parent == p && p instanceof ProcEnvironment)// is this a recursion?
      dp = ((ProcEnvironment)dp).dynamicParent;	// then don't leave calling frame in dynamic path
    parent = p; dynamicParent = dp;
  }

  // quicky!
  public ProcEnvironment(Environment p, Object argnames, Cons argvals) {
    parent = p;
    names = argnames;
    values = argvals;
  }

  public ProcEnvironment(Environment p, Environment dp, Object argnames, Cons argvals) {
    // here's a kludge to prevent frame build-up
    if (dp.parent == p && p instanceof ProcEnvironment)// is this a recursion?
      dp = ((ProcEnvironment)dp).dynamicParent;	// then don't leave calling frame in dynamic path
    parent = p;
    dynamicParent = dp;
    names = argnames;
    values = argvals;
  }

  public void addBinding(Symbol name, Object newValue) {
    try {
      setBinding(name, newValue, true);
    }
    catch (SchemeException e) {
      System.out.println("this can't happen"); // or so I hope
    }
  }

  public void setBinding(Symbol name, Object newValue) throws SchemeException {
    setBinding(name, newValue, false);
  }

  // does a define or set based on define argument
  public void setBinding(Symbol name, Object newValue, boolean define) throws SchemeException {
    Object restNames;
    Cons restValues;
    for (restNames = names, restValues = values;
	 restNames != Nil.nil; 
	 restNames = ((Cons)restNames).cdr, restValues = restValues.kdr())
      {
        if (restNames instanceof Symbol) {
	  if (restNames == name) {
	    // +++ leaves the old value around. This won't affect semantics but is a GC bug.
	    names = new Cons(name, names);
	    values = new Cons(newValue, values);
	    return;
	  }
	  else
	    break;
	}
	else if (((Cons)restNames).car == name) {
	  ((Cons)restValues).car = newValue;
	  return;
	}
      }

    if (define) {
      names = new Cons(name, names);
      values = new Cons(newValue, values);      
    }
    else {
      if (parent != null)
	parent.setBinding(name, newValue);
      else
	throw new SchemeException("can't set unbound variable " + name);
    }
  }
    
  public Object getBinding(Symbol name, int lookupType) throws SchemeException {
    Object restNames;
    Cons restValues;
    for (restNames = names, restValues = values;
	 restNames != Nil.nil; 
	 restNames = ((Cons)restNames).cdr, restValues = restValues.kdr())
      {
        if (restNames instanceof Symbol) {
	  if (restNames == name)
	    return restValues;
	  else
	    break;
	}
	else if (((Cons)restNames).car == name)
	  return restValues.car;
      }
    Environment up = null;	// extra set is necessary because of broken verifiers
    switch (lookupType) {
    case Environment.GET_REGULAR: up = parent; break;
    case Environment.GET_DYNAMIC: up = dynamicParent; break;
    case Environment.GET_LOCAL: up = null;
    }
    if (up != null)
      return up.getBinding(name, lookupType);
    else
      throw new SchemeException(name + " is unbound");
  }

  public String toString() {
    return "ProcEnvironment[" + getLocalBindings().toString() + "]";
  }

  public void forBindings(Procedure proc) throws SchemeException {
    for (Object restNames = names;
	 restNames != Nil.nil; 
	 restNames = ((Cons)restNames).cdr)
      {
        if (restNames instanceof Symbol) {
	  proc.apply(Environment.top, new Cons(restNames));
	  break;
	}
	else 
	  proc.apply(Environment.top, new Cons(((Cons)restNames).car));
      }
  }

  public TopEnvironment getTopEnvironment() {
    if (dynamicParent == null) {
      if (parent == null) 
	return null;
      else
	return parent.getTopEnvironment(); // if stuck, try going up lexical environment
    }
    else
      return dynamicParent.getTopEnvironment();
  }
}
