package com.ibm.jikes.skij;  
import com.ibm.jikes.skij.util.*;
import java.io.*;

/* This file is part of Skij.
 * Author: Michael Travers (mt@watson.ibm.com)
 * 
 * Licensed Materials - See the file license.txt.
 * (c) Copyright IBM Corp. 1997, 1998. All rights reserved.
 */

public class Scheme {

  final static String version = "1.4";
  final static String needsJDKVersion = "1.1";
  
  public static boolean initedp = false;

  public static PrintWriter out = new PrintWriter(System.out, true);

  // usually Scheme is not instantiated, but some startup modes use this
  public Scheme() {
    start();
  }

  public static boolean greetAndInitialize(PrintWriter out) {

    try {

      if (Invoke.compareVersionStrings(System.getProperty("java.version"),
				       needsJDKVersion)) {
	out.println("Welcome to Skij (Scheme in Java) version " + version);
	out.println("Please send comments to mt@watson.ibm.com");
	out.println("Running under Java version " + System.getProperty("java.version") + " from " + System.getProperty("java.vendor"));
	initLibraries();
	out.println("Initialized.");
	return true;
      }
    
      else {
	out.println("Running under Java version " + System.getProperty("java.version") + " from " + System.getProperty("java.vendor"));
	out.println("Sorry, you need at least Java" + needsJDKVersion + " to run Skij");
	return false;
      }
    }
    catch (Throwable e) {
      out.println("Error trying to verify version" + e.toString());
      return true;
    }
  }

  public static void main(String[] args) {

    String initForm = null;

    for (int i = 0; i < args.length; i++) {
      String arg = args[i];
      if (arg.equals("-help")) {
	System.out.println("-init <form>    eval form instead of starting listener");
	System.out.println("-trace          turn tracing on as in (trace #t), but takes effect early");
	System.exit(0); }
      if (arg.equals("-init")) 
	initForm = args[++i];
      if (arg.equals("-trace")) 
	Tracer.setOn(true);
      if (arg.equals("-nolibs"))
	javaLibraries = true;
    }
    start(initForm, false);
  }

  public static void start() {
    start(null, false);
  }

  public static void start(String initForm) {
    start(initForm, false);
  }
  
  public static void start(final String initForm, boolean threadp) {
    
    if (!greetAndInitialize(out))
      System.exit(-1);

    // process an initialization form, or start up listener
    // note kludge: some command lines don't allow spaces, so permit underscore substitute
    if (initForm == null)
      if (threadp) {
	new Thread(new SchemeListener()).start(); }
      else
	new SchemeListener().run();
    else			// initForm supplied
      if (threadp) {
	new Thread( new SchemeListener() {
	  public void run() {
	    Scheme.evalPrint(initForm.replace('_',' '));
	  }}).start();
      }
      else
	evalPrint(initForm.replace('_',' ')); 
  }

  public static void initLibraries() {
    if (!initedp) {
      Environment.initialize();      // set up global environment
      try {
	loadResource("lib/init.scm");
	initedp = true;
      }
      catch (Throwable e) {
	System.out.println("Can't find init file: " + e.toString());
      }
    }
  }

  // evaluate a string; handle errors
  public static Object evalString(String string) {
    initLibraries();
    try {
      return eval(new SchemeTokenizer(string).nextThing()); 
    }
    catch (Exception e) {
      System.out.println("Error during eval of " + string + ": " + e.toString());
      return null;
    }   
  }

  // evaluate a string and print result
  public static Object evalPrint(String string) {
    Object value = evalString(string);
    if (value != null) 
      System.out.println("Eval returned: " + value.toString());
    return value;
  }

  // the most basic evaluator
  public static Object eval(Object thing) throws SchemeException {
    return Evaluator.eval(thing, Environment.top);
  }

  // for outside callers
  public static void inspect(Object thing) {
    initLibraries();
    Scheme.procedure("inspect").safeTopApply(Cons.list(thing));
  }

  // for quick references to procedures in code
  // ex:  Scheme.procedure("write").apply(environment, Cons.list(result));
  public static Procedure procedure(String name) {
    Procedure proc;
    try {
      return (Procedure)Environment.top.getBinding(Symbol.intern(name));
    }
    catch (Throwable e) {
      System.out.println("Couldn't get procedure " + name + ": " + e.toString());
      return null;
    }
  }

  public static boolean javaLibraries = false;

  public static void loadResource(String name) throws SchemeException, ClassNotFoundException, IOException {
    if (javaLibraries) loadResourceJava(name);
    else {
      Class skijClass = Class.forName("com.ibm.jikes.skij.Scheme"); 	// any skij class will do
      InputStream s = skijClass.getResourceAsStream(name);
      if (s == null)
	throw new SchemeException("resource " + name + " not found");
      else {
	System.out.println("Loading resource " + name + "...");
	new SchemeListener(s).repl();
	procedure("provide-resource").safeTopApply(Cons.list(name));
	System.out.println("...finished loading resource " + name);
      }
    }
  }

  static void loadResourceJava(String name) throws SchemeException {
    // extract lib name (without extension)
    int pos = name.indexOf('.');
    String bname = name.substring(0, pos);
    try {
      Class.forName( "com.ibm.jikes.skij." + bname.replace('/', '.'));
    }
    catch (Throwable e) {
      throw new SchemeException("Couldn't load library " + name + ": " + e.toString());
    }
  }

  public static void loadURL(java.net.URL url) throws IOException, SchemeException {
    // System.out.println("Loading from " + url.toString());
    new SchemeListener(url.openStream()).repl();
  }
  
}


