;;; random control 

; doesn't support else
(defmacro (cond . clauses)
  (if (null? clauses) #f
      (if (null? (cdar clauses))
	  `(or ,(caar clauses)		;handle (cond ((foo)) ..) case
	       (cond ,@(cdr clauses)))
	  `(if ,(caar clauses)
	       (begin ,@(cdar clauses))
	       (cond ,@(cdr clauses))))))

;ok, now it does
(define else #t)

; not done yet
(defmacro (case item . clauses)
  (let ((sym '%%gensym%%))
    `(let ((,sym ,item))
       ,(cons 'cond (map (lambda (clause)
			   (cons (if (eq? (car clause) 'else)
				     #t
;;; This file is part of Skij.
;;; Author: Michael Travers (mt@watson.ibm.com)

;;; Licensed Materials - See the file license.txt.
;;; (c) Copyright IBM Corp. 1997, 1998. All rights reserved.

				     `(memv ,sym ',(car clause)))
				 (cdr clause)))
			 clauses)))))

; this should probably be a primitive but lets get it right first
; i imagine this is slower than equal? most of the time
(define (eqv? a b)
  (or (eq? a b)
      (and (instanceof a 'java.lang.Number)
	   (instanceof b 'java.lang.Number)
	   (= a b))
      (and (instanceof a 'java.lang.Character)
	   (instanceof b 'java.lang.Character)
	   (invoke a 'equals b))
      ))

(defmacro (repeat n . body)
  `(let loop ((i ,n))
     (unless (= i 0)
      ,@body
      (loop (- i 1)))))

(define (not x)
  (if x #f #t))

; Anaphoric if
(defmacro (aif pred then . else)
  `(let ((it ,pred))
     (if it
	 ,then
	 ,@else)))

(defmacro (aand . args)
  (cond ((null? args) #t)
        ((null? (cdr args)) (car args))
        (#t `(aif ,(car args) (aand ,@(cdr args)) #f))))

(defmacro (when pred . body)
  `(if ,pred (begin ,@body)))

(defmacro (unless pred . body)
  `(if (not ,pred) (begin ,@body)))

(defmacro (awhen pred . body)
  `(aif ,pred (begin ,@body)))

; Keyword argument handler
(defmacro (key args key default)
  `(define ,key (aif (memq ',key ,args)
		     (cadr it)
		     ,default)))

